
# Lookup the Orocos::RTT compile and link flags.

# Sets the following variables (example: OROCOS_TARGET = lxrt)
#
# OS_LXRT                (defined)      
# OROCOS-RTT_FOUND       (defined)
# OROCOS-RTT_INCLUDES    Absolute path to the include directory
# OROCOS-RTT_LIBS        Absolute path to the library
# OROCOS-RTT_DEFINITIONS The -DOROCOS_TARGET=lxrt macro
# OROCOS-RTT_VERSION     For the detected API version 
#
# For each component listed in find_package the variable OROCOS-RTT_<COMPONENT>_LIBS is defined as well.
# The component name is lower case (the same case as the library name), the variable name is upper case.
#
# Example Usage: find_package(Orocos-RTT 1.6 REQUIRED corba)
# See the cmake find_package help for all possible options.

include(FindPackageHandleStandardArgs)

MESSAGE( STATUS "Looking for RTT version v${Orocos-RTT_FIND_VERSION}" )
SET(OROCOS_TARGET "gnulinux" CACHE STRING "The (RT-)OS target we are compiling for. One of: gnulinux, macosx, lxrt, xenomai.")
SET(OROCOS_CORBA_TARGET "corba-${OROCOS_TARGET}" )
string(TOUPPER ${OROCOS_TARGET} OROCOS_TARGET_CAPS )
SET(OS_${OROCOS_TARGET_CAPS} 1)

find_path(OROCOS-RTT_INCLUDE_DIR rtt/os/targets/${OROCOS_TARGET} )
find_path(OROCOS-RTT_INCLUDE_DIR rtt/os/${OROCOS_TARGET}/${OROCOS_TARGET}.h )

# Extract version information.
IF(OROCOS-RTT_INCLUDE_DIR)
  # Extract OROCOS-RTT_VERSION and OROCOS-RTT_LIB_VERSION from rtt-config.h
  # Read the whole file:
  #
  SET(OROCOS-RTT_VERSION 0)
  SET(OROCOS-RTT_LIB_VERSION "")
  FILE(READ "${OROCOS-RTT_INCLUDE_DIR}/rtt/rtt-config.h" _OROCOS-RTT_RTT_CONFIG_HPP_CONTENTS)
  
  STRING(REGEX REPLACE ".*#define RTT_VERSION [ ]*([0-9]+.[0-9]+.[0-9]+).*" "\\1" OROCOS-RTT_VERSION "${_OROCOS-RTT_RTT_CONFIG_HPP_CONTENTS}")
  STRING(REGEX REPLACE ".*#define RTT_VERSION_MAJOR ([0-9]+).*" "\\1" OROCOS-RTT_VERSION_MAJOR "${_OROCOS-RTT_RTT_CONFIG_HPP_CONTENTS}")
  STRING(REGEX REPLACE ".*#define RTT_VERSION_MINOR ([0-9]+).*" "\\1" OROCOS-RTT_VERSION_MINOR "${_OROCOS-RTT_RTT_CONFIG_HPP_CONTENTS}")
  STRING(REGEX REPLACE ".*#define RTT_VERSION_PATCH ([0-9]+).*" "\\1" OROCOS-RTT_VERSION_PATCH "${_OROCOS-RTT_RTT_CONFIG_HPP_CONTENTS}")

  #MESSAGE( STATUS "Found       RTT version v${OROCOS-RTT_VERSION}" )
ENDIF(OROCOS-RTT_INCLUDE_DIR)

#  Check version number 
#  user's "1" is passed as "1.0.0"
#  user's "1.4" is passed as "1.4.0"
if ( ${OROCOS-RTT_VERSION} LESS ${Orocos-RTT_FIND_VERSION})
  MESSAGE(SEND_ERROR "rtt-config.h reports version ${OROCOS-RTT_VERSION} < required version ${Orocos-RTT_FIND_VERSION}")
endif()

find_library(OROCOS-RTT_LIBRARY orocos-rtt-${OROCOS_TARGET} )

set(OROCOS-RTT_INCLUDES ${OROCOS-RTT_INCLUDE_DIR} )
set(OROCOS-RTT_LIBS ${OROCOS-RTT_LIBRARY})
set(OROCOS-RTT_PROCESS_LIBS OROCOS-RTT_LIBRARY)

foreach (COMPONENT ${Orocos-RTT_FIND_COMPONENTS})
    string(TOUPPER ${COMPONENT} UCOMPONENT)
    # Find an RTT shared library or plugin
    find_library (OROCOS-RTT_${UCOMPONENT}_LIBRARY NAMES "orocos-rtt-${COMPONENT}-${OROCOS_TARGET}")
    list (APPEND OROCOS-RTT_LIBS ${OROCOS-RTT_${UCOMPONENT}_LIBRARY})
    list (APPEND OROCOS-RTT_PROCESS_LIBS OROCOS-RTT_${UCOMPONENT}_LIBRARY)
    if (OROCOS-RTT_${UCOMPONENT}_LIBRARY)
      set (OROCOS-RTT_${UCOMPONENT}_FOUND TRUE)
      list (APPEND OROCOS-RTT_FOUND_COMPONENTS ${COMPONENT})
    else ()
      set (OROCOS-RTT_${UCOMPONENT}_FOUND FALSE)
      set (OROCOS-RTT_FOUND FALSE)
      list (APPEND OROCOS-RTT_MISSING_COMPONENTS ${COMPONENT})
    endif ()
    mark_as_advanced (OROCOS-RTT_${UCOMPONENT}_LIBRARY)
endforeach ()

if (NOT OROCOS-RTT_MISSING_COMPONENTS)
  MESSAGE("Found these RTT Components: ${OROCOS-RTT_FOUND_COMPONENTS}")
else()
  MESSAGE("These RTT Components were missing: ${OROCOS-RTT_MISSING_COMPONENTS}")
endif()


find_library(OROCOS-RTT_CORBA_LIBRARY orocos-rtt-${OROCOS_CORBA_TARGET} )

find_package_handle_standard_args(Orocos-RTT DEFAULT_MSG ${OROCOS-RTT_PROCESS_LIBS} OROCOS-RTT_INCLUDES)

set(OROCOS-RTT_DEFINITIONS "OROCOS_TARGET=${OROCOS_TARGET}")