#include <rtt/TaskContext.hpp>
#include <rtt/Attribute.hpp>
#include <rtt/Method.hpp>
#include <rtt/Command.hpp>
#include <rtt/Ports.hpp>
#include <iostream>
#include <rtt/os/main.h>

#include <ocl/ComponentLoader.hpp>

using namespace Orocos;
using namespace std;

/**
 * Quick Orocos component building in C++
 */
class Quicky 
    : public RTT::TaskContext
{
    /**
     *  Data Ports.
     * See also the 'dataflow-task' example !
     */
    DataPort<double> indatPort;
    DataPort<double> outdatPort;

    /**
     *  Properties.
     */
    Property<double> value;

    /**
     *  Attributes
     */
    Attribute<double> max;

    /**
     *  Events.
     */
    // An event with a bool argument:
    Event< void(bool) > turnSwitch;

    /**
     *  Methods.
     */
    double changeParameter(double f) { double old = value; value = f; return old; }

    /**
     *  Commands.
     */
    /**
     * The first command starts a cycle.
     */
    bool startCommand() { cout <<"Command startCommand() called!"<<endl; return true; }
    bool startCommandDone() const { cout <<"Condition startCommandDone() const called!"<<endl; return true; }

public:
    /**
     * Constructor. Sets up the interface of this TaskContext.
     */
    Quicky(std::string name) 
        : RTT::TaskContext(name, PreOperational),
          indatPort("Data_R"),
          outdatPort("Data_W"),
          value("Value","Value Description",3.14),
           max("max",1.0), 
          turnSwitch("turnSwitch")
    {
        /**
         * Export ports to interface:
         */
        this->ports()->addPort( &indatPort );
        this->ports()->addPort( &outdatPort );

        /**
         * Method Factory Interface.
         */
        this->methods()->addMethod( method( "changeParameter", &Quicky::changeParameter, this),
                           "Change a parameter, return the old value.",
                           "New Value", "The new value for the parameter."  ); 

        /**
         * Attribute/Property Interface
         */
        this->attributes()->addAttribute( &max );
        this->properties()->addProperty( &value );

        /**
         * Command Interface
         */

        this->commands()->addCommand( command( "startCommand", &Quicky::startCommand,
                             &Quicky::startCommandDone, this),
                             "The Start Command."  );

        /**
         * Event Interface
         */
        this->events()->addEvent( &turnSwitch, "Emitted when a switch is turned.",
                                  "Value", "The new value of the switch: true or false."); 
    }
  
    /**
     * This function is for the configuration code.
     * Return false to abort configuration.
     */
    bool configureHook() {
        // ...
        return true;
    }

    /**
     * This function is for the application's startup code.
     * Return false to abort startup.
     */
    bool startHook() {
        // ...
        return true;
    }

    /**
     * This function is periodically called.
     */
    void updateHook() {
        // Your algorithm for periodic execution goes inhere
    }

    /**
     * This function is called when the task is stopped.
     */
    void stopHook() {
        // Your stop code after last updateHook()
    }

    /**
     * This function is called when the task is being deconfigured.
     */
    void cleanupHook() {
        // Your configuration cleanup code
    }
};


ORO_CREATE_COMPONENT(Quicky)
